open Assert
open Trees
;; stop_on_failure()


(* Some example trees *)
let t1 = 
  Node(Node(Empty,
	    2,
	    leaf 1),
       3,
       Node(leaf 3,
	    2,
	    leaf 1))

;; print_endline "--- t1 ---"
;; print_endline (string_of_tree t1)


let t3 = Node(Empty, 7,
	      Node(
		Node(
		  Node(
		    Node(
		      Empty,
		      3,
		      Empty),
		    4,
		    Empty),
		  5,
		  Empty),
		6,
		Empty))

;; print_endline "--- t3 ---"
;; print_endline (string_of_tree t3)



let t5 = 
  Node(Node(leaf 2, 1, Empty),
       0,
       Node(leaf 7, 6, leaf 8))
		    
;; print_endline "--- t5 ---"
;; print_endline (string_of_tree t5)


(* sums all of the integers appearing in a tree *)
let rec sum (t:tree) : int =
  begin match t with
	| Empty -> 0
(*	| Node (Empty, v, Empty) ->  no special leaf *)
  | Node (lt, v, rt) -> v + sum lt + sum rt
	end
	
let test () : bool = sum t1 = 12
;; run_test "sum t1" test	
let test () : bool = sum Empty = 0
;; run_test "sum Empty" test
let test () : bool = sum (leaf 2) = 2
;; run_test "sum (leaf 2)" test


(* counts the longest path from the root to a leaf node *)
let rec height (t:tree) : int =
  begin match t with
	| Empty -> 0
	| Node (lt, _, rt) -> 
		1 + max (height lt) (height rt)
	end
	
	
let test () : bool = height t1 = 3
;; run_test "height t1" test
let test () : bool = height Empty = 0
;;	 run_test "height Empty" test
let test () : bool = height (leaf 2) = 1 
;; run_test "height leaf" test


(* returns the in order traversal of the tree *)
let rec inorder (t:tree) : int list =
   begin match t with
	| Empty -> []
	| Node (lt, x, rt) ->
		let l1 = inorder lt in
		let l2 = inorder rt in
		l1 @ (x :: l2) 
	end


let test () : bool = inorder t1 = [2;1;3;3;2;1]
;; run_test "inorder t1" test
let test () : bool = inorder Empty = []
;;	 run_test "inorder Empty" test
let test () : bool = inorder (leaf 2) = [2] 
;; run_test "inorder leaf" test

(* returns the preorder traversal of the tree *)
let rec preorder (t:tree) : int list =
  failwith "preorder"

(* returns the post-order traversal of the tree *)
let rec postorder (t:tree) : int list =
  begin match t with
    | Empty -> []
    | Node(l,n,r) -> failwith "postorder"
  end


(* an pre-order search through an arbitrary tree.
   returns true if the tree contains n *)
let rec contains (t:tree) (n:int) : bool =
   begin match t with
	| Empty -> false
	| Node (lt, x, rt) -> n = x || contains lt n
	   || contains rt n
  end
	
let test () : bool = contains Empty 3 = false
;; run_test "contains Empty" test
let test () : bool = contains (leaf 2) 2 = true
;; run_test "contains leaf 2" test	
let test () : bool = contains (leaf 2) 3 = false
;; run_test "contains leaf 2" test	
let test () : bool = contains t1 1 = true
;; run_test "contains t1 1" test
